# © 2022 Le Filament (<http://www.le-filament.com>)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).

from odoo import tools
from odoo import models, fields, api


class AdefpatFormateurProjectReport(models.Model):
    _name = "adefpat.formateur.project.report"
    _description = "Rapport formateurs"
    _auto = False
    _order = 'formateur_id'

    # project fields
    project_id = fields.Many2one(
        comodel_name="project.project",
        string="Nom du dossier",
    )
    project_num = fields.Char(
        related="project_id.num_dossier", string="Numéro du dossier"
    )
    type_convention_id = fields.Many2one(
        comodel_name="adefpat.type.convention",
        string="Convention de financement",
        related="project_id.type_convention_id"
    )

    # formateur fields
    formateur_id = fields.Many2one(
        comodel_name="res.partner",
        string="Formateur",
    )
    formateur_name = fields.Char("Nom du formateur", related="formateur_id.name")
    number_hour_period = fields.Float("Nb heures sur période")

    # ------------------------------------------------------
    # Query
    # ------------------------------------------------------
    def _select(self):
        select_str = """
            SELECT
                row_number() over (ORDER BY pr.id) AS id,
                pr.id AS project_id,
                t.formateur_id AS formateur_id,
                SUM(t.duree_hr * 7) AS number_hour_period
            FROM
                project_task t
            LEFT JOIN
                project_project pr ON t.project_id = pr.id
            LEFT JOIN
                adefpat_type_convention c ON pr.type_convention_id = c.id
        """

        return select_str

    def _where(self):
        period_start = fields.Date.from_string(self.env.context.get("period_start"))
        period_end = fields.Date.from_string(self.env.context.get("period_end"))
        is_fd = self.env.context.get("is_fd")
        is_closed = self.env.context.get("is_closed")

        where = """
            WHERE
                t.duree_hr > 0
        """

        if period_start and period_end:
            where += """
                AND t.date_deadline >= '%s'
                AND t.date_deadline <= '%s'
        """ % (period_start, period_end)

        if is_fd:
            where += "AND c.is_fd is true "

        if is_closed:
            where += """
                AND pr.closing_date IS NOT NULL
                AND pr.closing_date <= '%s'
                AND pr.closing_date >= '%s'
            """ % (period_end, period_start)

        return where

    def _group_by(self):
        group_by = """
            GROUP BY
                pr.id, t.formateur_id
        """

        return group_by

    @api.model_cr
    def init(self):
        tools.drop_view_if_exists(self.env.cr, self._table)
        self.env.cr.execute("""CREATE or REPLACE VIEW %s as (
            %s %s %s
        )""" % (self._table, self._select(), self._where(), self._group_by()))
